rm(list = ls()) # clear memory
graphics.off()  # clear all plots
cat("\014")     # clear console (same as Ctrl-L in console)
set.seed(123)   # Set seed
par(mfrow = c(1,1))
#-------------------------------------------------------------------------------
# Credit to Mats Nielsson

#load libraries

library(tidyverse)
library(ltm)
library(psych)
library(lavaan)
library(semPlot)
library(semTools)
library(ltm)
library(mirt)

#IMPORT THE DATASET
df.ANX <- read.csv(file.choose(), sep = "\t", stringsAsFactors = T) #sep = \t might be not necessary in your computer
#retain only matching items
df.ANX <- df.ANX[, c("Group", "Age.T0", "Gender", "ANX_4", "ANX_5", "ANX_7", "ANX_8" )]


summary(df.ANX) # there are anomalous responses
# fix anomalous obs
ANX_ind <- grep(names(df.ANX), pattern = "^ANX") 
for(i in 1:length(ANX_ind)) {
    for(j in 1:nrow(df.ANX)){
        ifelse(df.ANX[j, ANX_ind[i]] > 5, df.ANX[j, ANX_ind[i]] <- NA, df.ANX[j,  ANX_ind[i]] <- df.ANX[j, ANX_ind[i]])
    }
}
df.ANX <- na.omit(df.ANX) # listwise removal (n = 2)
summary(df.ANX) # OK


#descriptives
describe(df.ANX) # describe
df.ANX%>% describeBy( group = df.ANX$Group) # describe by group

descr.gend <- tapply(df.ANX$Gender, df.ANX$Group, table) # describe gender by group
descr.gend


summary(df.ANX[, ANX_ind])
Hmisc::describe(df.ANX[, ANX_ind]) # describe frequencies

# Figure 1
pdf("Fig1.pdf", pointsize = 18)
par(mfrow=c(2,2))
par(mar=c(2,2,2,2)) # to avoid Error in plot.new() : figure margins too large
for(i in 1:length(ANX_ind)){
    hist(df.ANX[, ANX_ind[i]], col = "gray", xlab = "", main = paste("Item", i),  breaks = 0:5) # , main = 
}
par(mfrow=c(1,1))
dev.off()

#TCC based reliability
psych::alpha(df.ANX[, ANX_ind]) # cronbach's aplha

# confirmatory FA in lavaan
modelANX <-'
# measurement model
ANX =~ ANX_4 + ANX_5 + ANX_7 + ANX_8
'

fitCFANX <- cfa(modelANX, data = df.ANX[, ANX_ind], std.lv=TRUE,
                ordered=c("ANX_4", "ANX_5", "ANX_7", "ANX_8"))
summary(fitCFANX) 

fitMeasures(fitCFANX, fit.measures = c("srmr","rmsea", "tli","cfi", "wrmr")) #looks excellent for CFI, TLI and SRMR, not accptable for rmsea and wrmr
boot.fit <- parameterEstimates(fitCFANX, boot.ci.type="bca.simple",level=0.95, ci=TRUE,standardized = T)
boot.fit
reliability(fitCFANX) #omega 3 is McDonald0's

#IRT based EFA
cor.pol.ANX <- polychoric(df.ANX[, ANX_ind]) 
psych::alpha(cor.pol.ANX$rho) # polychoric Cronbach alpha

IRT.FA.ANX <- irt.fa(x = cor.pol.ANX, plot = TRUE, n.obs = nrow(df.ANX), fm = "minres") 
IRT.FA.ANX

# Figure 2
pdf("Fig2.pdf", pointsize = 18)
plot(IRT.FA.ANX$fa$e.values, 
     pch = 19, lty = 6, 
     ylab = "Eigenvalues", xlab = "Factors", main = "Scree Plot", xaxt='n')
# Now, define a custom axis
axis(side = 1, at=1:4)
lines(IRT.FA.ANX$fa$e.values)
dev.off()

(dsc <- descript(df.ANX[,ANX_ind])) #excellent crobnbach alpha, all the variables seem to be associated

# GRM IRT analysis
names(df.ANX)[ANX_ind] <- c("Item#1", "Item#2", "Item#3", "Item#4")

model1 <- mirt.model('ANX = 1-4
                    CONSTRAIN = (1-4, a1)')
model2 <- mirt.model('ANX = 1-4')
# Try different rasch models
mirt1 <- mirt(df.ANX[, ANX_ind], model1, itemtype = "graded") #constrained GRM

# estimate the unconstrained model
mirt2 <- mirt(df.ANX[, ANX_ind], model2, itemtype = "graded") #

coef(mirt1) 
coef(mirt2) 

plot(mirt1)
plot(mirt2)

(Table1 <- anova(mirt1,mirt2)) #Unsconstrained discr better than Rasch, also in terms of BIC
write.csv(Table1, file = "Table1.csv")

# M2() statistic for the goodness of fit 
M2_out <- M2(mirt2, type = "C2") # The C2 variant is mainly useful when polytomous response models do not have sufficient degrees of freedom to compute M2*.
#M2(mirt1, type = "C2") # The C2 variant is mainly useful when polytomous response models do not have sufficient degrees of freedom to compute M2*.
write.csv(M2_out, "M2.csv")
# Figure 3
# pdf("Fig3.pdf", pointsize = 24)
trellis.device(color=FALSE, device="pdf", file = "Fig3.pdf") # https://www.dummies.com/programming/r/how-to-print-and-save-a-lattice-plot-in-r/
plot3 <- plot(mirt1, type= "trace", 
     which.items = c(3, 4, 1, 2))
print(plot3) # https://stackoverflow.com/questions/20129725/r-code-doesnt-save-plot-image
dev.off()

# Figure 4
pdf("Fig4.pdf", pointsize = 18)
plot(mirt1, type= "rxx")
dev.off()

#Local Dependence
Theta <- fscores(mirt2, method = "MAP") #estimate thetas
residuals(mirt2, type = 'Q3', Theta=Theta) # many values with abs > .20 (7-9-11)
(above <- abs(residuals(mirt1, type = 'Q3', Theta=Theta)) > .2) # many values with abs > .20 (7-9-11)) 
(above2 <- (residuals(mirt1, type = 'Q3', Theta=Theta)) > .2) # many values with abs > .20 (7-9-11)) 

apply(above, 2, sum) # identify the most problematic items
apply(above2, 2, sum) - 1 # identify the most problematic item

#DIF
# ordinal logistic regression approach by Zumbo 1999
library(MASS)
dif.gend <- sapply(1:4, function(x) 
    car::Anova(polr(as.ordered(df.ANX[, ANX_ind[x]]) ~ Theta * Gender, data = df.ANX, Hess=TRUE))[2:3,3])
colnames(dif.gend) <- names(df.ANX[ANX_ind])
round(dif.gend, 3)

df.ANX$AgeGroup <- as.factor(ifelse(df.ANX$Age.T0 > 64,"OLD", "YOU"))

dif.age <- sapply(1:4, function(x) 
    car::Anova(polr(as.ordered(df.ANX[, ANX_ind[x]]) ~ Theta * AgeGroup, data = df.ANX, Hess=TRUE))[2:3,3])
colnames(dif.age) <- names(df.ANX[ANX_ind])
round(dif.age, 3) # if the apply function does not work, execute step bu step (e.g. car::Anova(polr(as.ordered(df.ANX[, ANX_ind[1]]) ~ Theta * AgeGroup, data = df.ANX, Hess=TRUE))) 
car::Anova(polr(as.ordered(df.ANX[, ANX_ind[1]]) ~ Theta * AgeGroup, data = df.ANX, Hess=TRUE))
car::Anova(polr(as.ordered(df.ANX[, ANX_ind[2]]) ~ Theta * AgeGroup, data = df.ANX, Hess=TRUE))
car::Anova(polr(as.ordered(df.ANX[, ANX_ind[3]]) ~ Theta * AgeGroup, data = df.ANX, Hess=TRUE))
car::Anova(polr(as.ordered(df.ANX[, ANX_ind[4]]) ~ Theta * AgeGroup, data = df.ANX, Hess=TRUE))
tapply(df.ANX[, ANX_ind[3]], df.ANX$AgeGroup, summary)

df.ANX$Group2 <- as.factor(ifelse(df.ANX$Group == "CTR", "CTR", "PAT"))
dif.group <- sapply(1:4, function(x) 
    car::Anova(polr(as.ordered(df.ANX[, ANX_ind[x]]) ~ Theta * Group2, data = df.ANX, Hess=TRUE))[2:3,3])
colnames(dif.group) <- names(df.ANX[ANX_ind])
round(dif.group, 3)

car::Anova(polr(as.ordered(df.ANX[, ANX_ind[1]]) ~ Theta * Group2, data = df.ANX, Hess=TRUE))
car::Anova(polr(as.ordered(df.ANX[, ANX_ind[2]]) ~ Theta * Group2, data = df.ANX, Hess=TRUE))
car::Anova(polr(as.ordered(df.ANX[, ANX_ind[3]]) ~ Theta * Group2, data = df.ANX, Hess=TRUE))
car::Anova(polr(as.ordered(df.ANX[, ANX_ind[4]]) ~ Theta * Group2, data = df.ANX, Hess=TRUE))
tapply(df.ANX[, ANX_ind[3]], df.ANX$Group2, summary)

MG.Gr <- multipleGroup(df.ANX[, c('ANX_4', 'ANX_5', 'ANX_7', 'ANX_8')], model2, group = df.ANX$Group2, SE = T, invariance = 'ANX_4')
plot(MG.Gr)

# invariance constraint as suggested here: https://groups.google.com/forum/#!msg/mirt-package/KxxiSU6hLp4/YD6L1pzLCAAJ

# Tscores stats
conversion <- fscores(mirt2, method = "MAP", full.scores = F) # conversion
write.csv(conversion, "ConversionTable.csv")

df.ANX$Tscores <- 50 + Theta*10
df.ANX$raw_scores <- rowSums(df.ANX[, ANX_ind])
cor(df.ANX$raw_scores, df.ANX$Tscores)
cor(df.ANX$raw_scores, df.ANX$Tscores)^2

plot(df.ANX$raw_scores, df.ANX$Tscores, pch = 19, ylab = "T Score", xlab = "Raw Score")

m1 <- lm(Tscores ~ Group + Age.T0 + Gender, data = df.ANX)
car::Anova(m1, type = "III")
contrasts(df.ANX$Group)
summary(m1)

m2 <- lm(raw_scores ~ Group + Age.T0 + Gender, data = df.ANX)
car::Anova(m2, type = "III")
summary(m2)

library(emmeans)
lsmeans(m1, list(pairwise ~ Group), adjust = "bonferroni")
lsmeans(m2, list(pairwise ~ Group), adjust = "bonferroni")

with(df.ANX, tapply(Tscores, Group, mean))%>%round(2)
with(df.ANX, tapply(Tscores, Group, sd))%>%round(2)
descr.group <- df.ANX  %>% dplyr::select(Group, Tscores) %>% describeBy(group = df.ANX$Group, mat = T) # describe by group
descr.group2 <- descr.group[5:8,c(-1, -3, -8, -9, -15) ]
names(descr.group2)[1] <- "Group"
descr.group2$Group <- factor(descr.group2$Group)
row.names(descr.group2) <- levels(descr.group2$Group)
descr.group2
write.csv(descr.group2, "DescriptiveTscores.csv")

pdf("Fig5.pdf", pointsize = 18)
plot(lsmeans(m1, ~ Group))
dev.off()

plot(lsmeans(m2, ~ Group))
